@@ -8,39 +8,71 @@ Resolwe
    :members:
 
 """
+
 import getpass
+import json
 import logging
 import ntpath
 import os
 import re
-from urllib.parse import urljoin
+import time
+import webbrowser
+from collections import defaultdict
+from contextlib import suppress
+from importlib.metadata import version as package_version
+from pathlib import Path
+from typing import Iterable, Optional, TypedDict, Union
+from urllib.parse import urlencode, urljoin, urlparse
 
 import requests
 import slumber
+import tqdm
+from packaging import version
 
 from resdk.uploader import Uploader
 
 from .constants import CHUNK_SIZE
-from .exceptions import ValidationError, handle_http_exception
-from .query import ResolweQuery
+from .exceptions import ResolweServerError, ValidationError, handle_http_exception
+from .query import (
+    AnnotationFieldQuery,
+    AnnotationValueQuery,
+    PredictionFieldQuery,
+    PredictionValueQuery,
+    ResolweQuery,
+)
 from .resources import (
+    AnnotationField,
+    AnnotationValue,
     Collection,
     Data,
     DescriptorSchema,
     Geneset,
     Group,
     Metadata,
+    PredictionField,
+    PredictionGroup,
+    PredictionPreset,
+    PredictionValue,
     Process,
     Relation,
     Sample,
     User,
+    Variant,
+    VariantAnnotation,
+    VariantCall,
+    VariantExperiment,
 )
 from .resources.base import BaseResource
+from .resources.fields import DataSource
 from .resources.kb import Feature, Mapping
 from .resources.utils import get_collection_id, get_data_id, is_data, iterate_fields
-from .utils import is_email
+from .utils import md5
 
 DEFAULT_URL = "http://localhost:8000"
+AUTOMATIC_LOGIN_POSTFIX = "saml-auth/api-login/"
+INTERACTIVE_LOGIN_POSTFIX = "saml-auth/remote-login/"
+MINIMAL_SUPPORTED_VERSION_POSTFIX = "api/resdk_minimal_supported_version"
+SERVER_MODULE_VERSIONS_POSTFIX = "/about/versions"
 
 
 class ResolweResource(slumber.Resource):
@@ -53,6 +85,21 @@ class ResolweResource(slumber.Resource):
             return handle_http_exception(attr)
         return attr
 
+    def delete(self, *args, **kwargs):
+        """Delete resource object.
+
+        This is mostly Slumber default implementation except that it returns the
+        processed response when status is not 204 (No Content).
+        """
+        resp = self._request("DELETE", params=kwargs)
+        if 200 <= resp.status_code <= 299:
+            if resp.status_code == 204:
+                return True
+            else:
+                return self._process_response(resp)
+        else:
+            return False
+
 
 class ResolweAPI(slumber.API):
     """Use custom ResolweResource resource class in slumber's API."""
@@ -63,7 +110,7 @@ class ResolweAPI(slumber.API):
 class Resolwe:
     """Connect to a Resolwe server.
 
-    :param username: user's username
+    :param username: user's email
     :type username: str
     :param password: user's password
     :type password: str
@@ -72,20 +119,39 @@ class Resolwe:
 
     """
 
+    # Map between resource and Query map. Default in ResorweQuery, only overrides must
+    # be listed here.
+    resource_query_class = {
+        AnnotationField: AnnotationFieldQuery,
+        AnnotationValue: AnnotationValueQuery,
+        PredictionField: PredictionFieldQuery,
+        PredictionValue: PredictionValueQuery,
+    }
+
     # Map resource class to ResolweQuery name
     resource_query_mapping = {
-        Data: "data",
+        AnnotationField: "annotation_field",
+        AnnotationValue: "annotation_value",
         Collection: "collection",
-        Sample: "sample",
-        Relation: "relation",
-        Process: "process",
+        Data: "data",
         DescriptorSchema: "descriptor_schema",
-        User: "user",
-        Group: "group",
         Feature: "feature",
-        Mapping: "mapping",
         Geneset: "geneset",
+        Group: "group",
+        Mapping: "mapping",
         Metadata: "metadata",
+        PredictionField: "prediction_field",
+        PredictionGroup: "prediction_group",
+        PredictionPreset: "prediction_preset",
+        PredictionValue: "prediction_value",
+        Process: "process",
+        Relation: "relation",
+        Sample: "sample",
+        User: "user",
+        Variant: "variant",
+        VariantAnnotation: "variant_annotation",
+        VariantCall: "variant_calls",
+        VariantExperiment: "variant_experiment",
     }
     # Map ResolweQuery name to it's slug_field
     slug_field_mapping = {
@@ -113,8 +179,14 @@ class Resolwe:
 
     session = None
 
-    def __init__(self, username=None, password=None, url=None):
+    def __init__(
+        self,
+        username: Optional[str] = None,
+        password: Optional[str] = None,
+        url: Optional[str] = None,
+    ):
         """Initialize attributes."""
+        self.logger = logging.getLogger(__name__)
         self.session = requests.Session()
         self.uploader = Uploader(self)
         if url is None:
@@ -130,11 +202,49 @@ class Resolwe:
             password = os.environ.get("RESOLWE_API_PASSWORD", None)
 
         self.url = url
+
+        # Check minimal supported version.
+        self.version_check()
+
         self._login(username=username, password=password)
 
-        self.logger = logging.getLogger(__name__)
+    def version_check(self):
+        """Check that the server is compatible with the client."""
+        url = urljoin(self.url, MINIMAL_SUPPORTED_VERSION_POSTFIX)
+        try:
+            response = requests.get(url)
+            minimal_version = version.parse(
+                response.json()["minimal_supported_version"]
+            )
+            my_version = version.parse(package_version("resdk"))
+            if my_version < minimal_version:
+                message = (
+                    f"Warning: your version of ReSDK ('{my_version}') is not compatible with "
+                    f"the server: minimal supported version is '{minimal_version}'. "
+                    "To update the package run\n\n"
+                    "python -m pip install --upgrade resdk\n\n"
+                    "from the command line."
+                )
+                self.logger.warning(message)
+        except Exception:
+            self.logger.warning(
+                "Warning: unable to read the minimal supported version from the server."
+            )
 
-    def _validate_url(self, url):
+    def version_output(self) -> dict:
+        """Output the version of the server modules."""
+        url = urljoin(self.url, SERVER_MODULE_VERSIONS_POSTFIX)
+        try:
+            response = requests.get(url)
+        except requests.exceptions.RequestException:
+            raise ResolweServerError("Unable to read the server version.")
+        return response.json()
+
+    def _validate_url(self, url: str):
+        """Validate server URL.
+
+        :raises ValueError: if the URL is not valid or the server is unreachable.
+        """
         if not re.match(r"https?://", url):
             raise ValueError("Server url must start with http(s)://")
 
@@ -147,13 +257,19 @@ class Resolwe:
         """Initialize ResolweQuery's."""
         for resource, query_name in self.resource_query_mapping.items():
             slug_field = self.slug_field_mapping.get(query_name, "slug")
-            query = ResolweQuery(self, resource, slug_field=slug_field)
+            QueryClass = self.resource_query_class.get(resource, ResolweQuery)
+            query = QueryClass(self, resource, slug_field=slug_field)
             if query_name in self.query_filter_mapping:
                 query = query.filter(**self.query_filter_mapping[query_name])
             setattr(self, query_name, query)
 
-    def _login(self, username=None, password=None):
-        self.auth = ResAuth(username, password, self.url)
+    def _login(
+        self,
+        username: Optional[str] = None,
+        password: Optional[str] = None,
+        interactive: bool = False,
+    ):
+        self.auth = ResAuth(username, password, self.url, interactive=interactive)
         self.session.cookies = requests.utils.cookiejar_from_dict(self.auth.cookies)
         self.api = ResolweAPI(
             urljoin(self.url, "/api/"),
@@ -164,20 +280,27 @@ class Resolwe:
         self._initialize_queries()
         self.uploader.invalidate_cache()
 
-    def login(self, username=None, password=None):
-        """Interactive login.
+        # Retrieve the logged in user and save it to auth. Necessary for interactive
+        # login.
+        with suppress(Exception):
+            logged_in_user = self.user.get(current_only=True)
+            self.auth.username = logged_in_user.email
 
-        Ask the user to enter credentials in command prompt. If
-        username / email and password are given, login without prompt.
+    def login(self, username: Optional[str] = None, password: Optional[str] = None):
+        """Perform the interactive login.
+
+        If only username is given prompt the user for password via shell.
+        If username is not given, prompt for interactive login.
         """
-        if username is None:
-            username = input("Username (or email): ")
-        if password is None:
+        if username is not None and password is None:
             password = getpass.getpass("Password: ")
-        self._login(username=username, password=password)
+        self._login(username=username, password=password, interactive=True)
 
-    def get_query_by_resource(self, resource):
-        """Get ResolweQuery for a given resource."""
+    def get_query_by_resource(self, resource: type[BaseResource]) -> ResolweQuery:
+        """Get ResolweQuery for a given resource.
+
+        :raises ValueError: if the resource is not a subclass of BaseResource.
+        """
         if isinstance(resource, BaseResource):
             resource = resource.__class__
         elif not issubclass(resource, BaseResource):
@@ -187,7 +310,7 @@ class Resolwe:
 
         return getattr(self, self.resource_query_mapping.get(resource))
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """Return string representation of the current object."""
         if self.auth.username:
             return "Resolwe <url: {}, username: {}>".format(
@@ -195,7 +318,7 @@ class Resolwe:
             )
         return "Resolwe <url: {}>".format(self.url)
 
-    def _process_file_field(self, path):
+    def _process_file_field(self, path: Union[str, Path, dict]) -> dict:
         """Process file field and return it in resolwe-specific format.
 
         Upload referenced file if it is stored locally and return
@@ -230,14 +353,14 @@ class Resolwe:
             "file_temp": file_temp,
         }
 
-    def _get_process(self, slug=None):
+    def _get_process(self, slug: Optional[str] = None) -> Process:
         """Return process with given slug.
 
         Raise error if process doesn't exist or more than one is returned.
         """
         return self.process.get(slug=slug)
 
-    def _process_inputs(self, inputs, process):
+    def _process_inputs(self, inputs: dict, process: Process) -> dict:
         """Process input fields.
 
         Processing includes:
@@ -304,14 +427,14 @@ class Resolwe:
 
     def run(
         self,
-        slug=None,
-        input={},
-        descriptor=None,
-        descriptor_schema=None,
-        collection=None,
-        data_name="",
-        process_resources=None,
-    ):
+        slug: Optional[str] = None,
+        input: dict = {},
+        descriptor: Optional[dict] = None,
+        descriptor_schema: Optional[str] = None,
+        collection: Optional[Collection] = None,
+        data_name: str = "",
+        process_resources: Optional[dict] = None,
+    ) -> Data:
         """Run process and return the corresponding Data object.
 
         1. Upload files referenced in inputs
@@ -366,9 +489,9 @@ class Resolwe:
             data["process_resources"] = process_resources
 
         model_data = self.api.data.post(data)
-        return Data(resolwe=self, **model_data)
+        return Data(resolwe=self, **model_data, initial_data_source=DataSource.SERVER)
 
-    def get_or_run(self, slug=None, input={}):
+    def get_or_run(self, slug: Optional[str] = None, input: dict = {}):
         """Return existing object if found, otherwise create new one.
 
         :param str slug: Process slug (human readable unique identifier)
@@ -378,24 +501,27 @@ class Resolwe:
         inputs = self._process_inputs(input, process)
 
         data = {
-            "process": process.slug,
+            "process": {"slug": process.slug},
             "input": inputs,
         }
 
         model_data = self.api.data.get_or_create.post(data)
-        return Data(resolwe=self, **model_data)
+        return Data(resolwe=self, **model_data, initial_data_source=DataSource.SERVER)
 
-    def _download_files(self, files, download_dir=None):
+    def _download_files(
+        self,
+        files: Iterable[Union[str, Path]],
+        download_dir: Union[str, None] = None,
+        show_progress: bool = True,
+    ):
         """Download files.
 
         Download files from the Resolwe server to the download
         directory (defaults to the current working directory).
 
         :param files: files to download
-        :type files: list of file URI
         :param download_dir: download directory
-        :type download_dir: string
-        :rtype: None
+            If not specified, the current working directory is used.
 
         """
         if not download_dir:
@@ -408,9 +534,12 @@ class Resolwe:
 
         if not files:
             self.logger.info("No files to download.")
-
         else:
             self.logger.info("Downloading files to %s:", download_dir)
+            # Store the sizes of files in the given directory.
+            # Use the dictionary to cache the responses.
+            sizes: dict[str, dict[str, int]] = defaultdict(lambda: defaultdict(int))
+            checksums: dict[str, dict[str, int]] = defaultdict(lambda: defaultdict(int))
 
             for file_uri in files:
                 file_name = os.path.basename(file_uri)
@@ -425,9 +554,27 @@ class Resolwe:
 
                 self.logger.info("* %s", os.path.join(file_path, file_name))
 
-                with open(
-                    os.path.join(download_dir, file_path, file_name), "wb"
-                ) as file_handle:
+                file_directory = os.path.dirname(file_url)
+                if file_directory not in sizes:
+                    content = self.session.get(file_directory, auth=self.auth).content
+                    for entry in json.loads(content):
+                        if entry["type"] != "file":
+                            continue
+                        sizes[file_directory][entry["name"]] = entry["size"]
+                        checksums[file_directory][entry["name"]] = entry["md5"]
+
+                file_size = sizes[file_directory][file_name]
+
+                with (
+                    tqdm.tqdm(
+                        total=file_size,
+                        disable=not show_progress,
+                        desc=f"Downloading file {file_name}",
+                    ) as progress_bar,
+                    open(
+                        os.path.join(download_dir, file_path, file_name), "wb"
+                    ) as file_handle,
+                ):
                     response = self.session.get(file_url, stream=True, auth=self.auth)
 
                     if not response.ok:
@@ -435,6 +582,19 @@ class Resolwe:
                     else:
                         for chunk in response.iter_content(chunk_size=CHUNK_SIZE):
                             file_handle.write(chunk)
+                            progress_bar.update(len(chunk))
+
+                # Verify md5 checksum:
+                if file_name.endswith(".html"):
+                    # Due to backend processing, html file fields have
+                    # checksums that are difficult to reproduce here.
+                    continue
+                expected_md5 = checksums[file_directory][file_name]
+                computed_md5 = md5(os.path.join(download_dir, file_path, file_name))
+                if expected_md5 != computed_md5:
+                    raise ValueError(
+                        f"Checksum ({computed_md5}) of downloaded file {file_name} does not match the expected value of {expected_md5}."
+                    )
 
     def data_usage(self, **query_params):
         """Get per-user data usage information.
@@ -446,58 +606,130 @@ class Resolwe:
         return self.api.base.data_usage.get(**query_params)
 
 
+class AuthCookie(TypedDict):
+    """Authentication cookie dict."""
+
+    csrftoken: str
+    sessionid: str
+
+
 class ResAuth(requests.auth.AuthBase):
     """HTTP Resolwe Authentication for Request object.
 
-    :param str username: user's username
+    :param str username: user's email
     :param str password: user's password
     :param str url: Resolwe server address
+    :param str cookies: user's sessionid and csrftoken cookies
 
     """
 
-    #: Session ID used in HTTP requests
-    sessionid = None
-    #: CSRF token used in HTTP requests
-    csrftoken = None
+    #: Dictionary of authentication cookes.
+    cookies: AuthCookie = {"csrftoken": "", "sessionid": ""}
 
-    def __init__(self, username=None, password=None, url=DEFAULT_URL):
+    def __init__(
+        self,
+        username: Optional[str] = None,
+        password: Optional[str] = None,
+        url: str = DEFAULT_URL,
+        interactive: bool = False,
+    ):
         """Authenticate user on Resolwe server."""
         self.logger = logging.getLogger(__name__)
-        self.cookies = {}
-
         self.username = username
         self.url = url
+        self.automatic_login_url = urljoin(self.url, AUTOMATIC_LOGIN_POSTFIX)
+        self.interactive_login_url = urljoin(self.url, INTERACTIVE_LOGIN_POSTFIX)
 
-        if not username and not password:
+        if not interactive and (username is None or password is None):
+            # Anonymous authentication
             return
 
-        key = "email" if is_email(username) else "username"
-        payload = {key: username, "password": password}
+        if username and password:
+            self.cookies = self.automatic_login(username, password)
+            self.logger.info(f"Successfully logged in as {username}.")
+        else:
+            self.cookies = self.interactive_login()
+            self.logger.info("Successfully logged in.")
 
-        try:
-            response = requests.post(urljoin(url, "/rest-auth/login/"), data=payload)
-        except requests.exceptions.ConnectionError:
-            raise ValueError("Server not accessible on {}. Wrong url?".format(url))
+    def automatic_login(self, username: str, password: str) -> AuthCookie:
+        """Attempt to perform automatic SAML login.
 
-        status_code = response.status_code
-        if status_code in [400, 403]:
-            msg = "Response HTTP status code {}. Invalid credentials?".format(
-                status_code
-            )
-            raise ValueError(msg)
+        :returns: authentication cookie dict on success, None on failure.
+        """
+        self.logger.info("Attempting automatic login.")
+        response = requests.post(
+            self.automatic_login_url,
+            data={"email": username, "password": password},
+        )
+        cookies = response.cookies.get_dict()
+        # Status should be either 204 (No Content) or 200 (OK).
+        if (
+            response.status_code not in [200, 204]
+            or "sessionid" not in cookies
+            or "csrftoken" not in cookies
+        ):
+            raise RuntimeError("Automatic login failed.")
 
-        if not ("sessionid" in response.cookies and "csrftoken" in response.cookies):
-            raise Exception("Missing sessionid or csrftoken. Invalid credentials?")
+        return {
+            "sessionid": cookies["sessionid"],
+            "csrftoken": cookies["csrftoken"],
+        }
 
-        self.sessionid = response.cookies["sessionid"]
-        self.csrftoken = response.cookies["csrftoken"]
-        self.url = url
-        self.cookies = {"csrftoken": self.csrftoken, "sessionid": self.sessionid}
+    def interactive_login(self, polling_interval: int = 1) -> AuthCookie:
+        """Prompt user to log in with a web browser.
 
-    def __call__(self, request):
+        :returns: authentication cookie dict on success, None on failure.
+        """
+        auth_id_url = urljoin(self.interactive_login_url, "auth-id/")
+        auth_id = requests.get(auth_id_url).json()["auth_id"]
+
+        # Use login url without the auth_id, as the system call could be intercepted.
+        browser_opened = webbrowser.open(self.interactive_login_url)
+        login_url_with_auth_id = (
+            urlparse(self.interactive_login_url)
+            ._replace(query=urlencode({"auth_id": auth_id}))
+            .geturl()
+        )
+        message_automatic = f"""Enter the following code in the opened browser window:
+
+{auth_id}
+
+Alternatively, you may visit the following URL which will autofill the code upon loading:
+{login_url_with_auth_id}\n"""
+        message_not_automatic = f"""Open the following URL which will autofill the code upon loading:
+
+{login_url_with_auth_id}\n"""
+
+        message = f"Browser {'could not' if not browser_opened else 'will'} be automatically opened.\n"
+        message += message_automatic if browser_opened else message_not_automatic
+
+        # Do not use logger here, because we want the url to be visible even if logging
+        # is disabled.
+        print(message)
+
+        poll_url = urljoin(self.interactive_login_url, "poll/")
+        session = requests.Session()
+        session.cookies.set("auth_id", auth_id)
+        response = session.get(poll_url)
+        while response.status_code == 204:
+            time.sleep(polling_interval)
+            response = session.get(poll_url)
+
+        cookie_dict = response.json()
+
+        if (
+            response.status_code != 200
+            or "sessionid" not in cookie_dict
+            or "csrftoken" not in cookie_dict
+        ):
+            raise RuntimeError("Interactive login failed.")
+
+        return cookie_dict
+
+    def __call__(self, request: requests.Request) -> requests.Request:
         """Set request headers."""
-        if self.sessionid and self.csrftoken:
-            request.headers["X-CSRFToken"] = self.csrftoken
+        if "csrftoken" in self.cookies:
+            request.headers["X-CSRFToken"] = self.cookies["csrftoken"]
 
         request.headers["referer"] = self.url
 
