@@ -10,31 +10,26 @@ Base Tables
     .. automethod:: __init__
 
 """
+
 import abc
 import asyncio
 import json
 import os
 import warnings
-from collections import Counter
+from collections import Counter, defaultdict
 from functools import lru_cache
 from io import BytesIO
+from pathlib import Path
 from typing import Callable, Dict, List, Optional
-from urllib.parse import urljoin
+from urllib.parse import urljoin, urlparse
 
 import aiohttp
 import pandas as pd
 import pytz
 from tqdm import tqdm
 
-from resdk.resolwe import Resolwe
 from resdk.resources import Collection, Data, Sample
-from resdk.resources.utils import iterate_schema
-from resdk.utils.table_cache import (
-    cache_dir_resdk,
-    clear_cache_dir_resdk,
-    load_pickle,
-    save_pickle,
-)
+from resdk.utils.table_cache import cache_dir_resdk, load_pickle, save_pickle
 
 # See _download_data function for in-depth explanation of this.
 EXP_ASYNC_CHUNK_SIZE = 50
@@ -60,14 +55,11 @@ class BaseTables(abc.ABC):
 
     process_type = None
     META = "meta"
-    QC = "qc"
 
     SAMPLE_FIELDS = [
         "id",
         "slug",
         "name",
-        "descriptor",
-        "descriptor_schema",
     ]
     DATA_FIELDS = [
         "id",
@@ -116,19 +108,13 @@ class BaseTables(abc.ABC):
         """
         return self._load_fetch(self.META)
 
-    @property
-    @lru_cache()
-    def qc(self) -> pd.DataFrame:
-        """Return samples QC table as a pandas DataFrame object.
-
-        :return: table of QC values
-        """
-        return self._load_fetch(self.QC)
-
-    @staticmethod
-    def clear_cache() -> None:
+    def clear_cache(self) -> None:
         """Remove ReSDK cache files from the default cache directory."""
-        clear_cache_dir_resdk()
+        # Clear cache for the collection
+        cache_dir = cache_dir_resdk()
+        for f in Path(cache_dir).iterdir():
+            if f.name.startswith(self.collection.slug):
+                f.unlink()
 
     @property
     @lru_cache()
@@ -142,7 +128,7 @@ class BaseTables(abc.ABC):
         """
         sample_ids = set([d.sample.id for d in self._data])
 
-        query = self.collection.samples.filter(fields=self.SAMPLE_FIELDS)
+        query = self.collection.samples.filter(fields=self.SAMPLE_FIELDS).iterate()
         return [s for s in query if s.id in sample_ids]
 
     @property
@@ -169,19 +155,20 @@ class BaseTables(abc.ABC):
 
         :return: list of Data objects
         """
-        data = []
-        sample_ids, repeated_sample_ids = set(), set()
+        sample2data = {}
+        repeated_sample_ids = set()
         for datum in self.collection.data.filter(
             type=self.process_type,
             status="OK",
-            ordering="-created",
             fields=self.DATA_FIELDS,
-        ):
-            if datum.sample.id in sample_ids:
+            # Do not include data that does not belong to a sample
+            entity__isnull=False,
+        ).iterate():
+            # We are using iterate to prevent 504 Bad Gateways
+            # This means that data is given from oldest to newest
+            if datum.sample.id in sample2data:
                 repeated_sample_ids.add(datum.sample.id)
-                continue
-            sample_ids.add(datum.sample.id)
-            data.append(datum)
+            sample2data[datum.sample.id] = datum
 
         if repeated_sample_ids:
             repeated = ", ".join(map(str, repeated_sample_ids))
@@ -191,7 +178,7 @@ class BaseTables(abc.ABC):
                 UserWarning,
             )
 
-        return data
+        return list(sample2data.values())
 
     @property
     @lru_cache()
@@ -201,9 +188,10 @@ class BaseTables(abc.ABC):
         The versioning of metadata on the server is determined by the
         newest of these values:
 
-            - newset modified sample
-            - newset modified relation
-            - newset modified orange Data
+            - newest modified sample
+            - newest modified relation
+            - newest modified orange Data
+            - newest modified AnnotationValue
 
         :return: metadata version
         """
@@ -214,6 +202,7 @@ class BaseTables(abc.ABC):
             "limit": 1,
         }
 
+        # Get newest sample timestamp
         try:
             newest_sample = self.collection.samples.get(**kwargs)
             timestamps.append(newest_sample.modified)
@@ -222,52 +211,53 @@ class BaseTables(abc.ABC):
                 f"Collection {self.collection.name} has no samples!"
             ) from None
 
+        # Get newest relation timestamp
         try:
             newest_relation = self.collection.relations.get(**kwargs)
             timestamps.append(newest_relation.modified)
         except LookupError:
             pass
 
+        # Get newest orange object timestamp
         try:
             orange = self._get_orange_object()
             timestamps.append(orange.modified)
         except LookupError:
             pass
 
+        # Get newest AnnotationValue timestamp
+        try:
+            newest_stamps = []
+            # Getting annotation values for large collection breaks the server.
+            # Instead make smaller queries for a single batch of samples
+            batch_size = 100
+            for i in self.tqdm(
+                range(0, len(self._samples), batch_size),
+                desc="Getting latest annotation_value timestamp",
+            ):
+                batch_sample_ids = [s.id for s in self._samples[i : i + batch_size]]
+                newest_ann_value = self.resolwe.annotation_value.get(
+                    entity__in=batch_sample_ids,
+                    ordering="-created",
+                    limit=1,
+                )
+                newest_stamps.append(newest_ann_value.modified)
+
+            timestamps.append(sorted(newest_stamps)[-1])
+
+        except LookupError:
+            pass
+
         newest_modified = sorted(timestamps)[-1]
         # transform into UTC so changing timezones won't effect cache
         version = (
             newest_modified.astimezone(pytz.utc).isoformat().replace("+00:00", "Z")
         )
+        # On Windows, datetime stamps are not appropriate as a part of file name.
+        # The reason is the colon char (":")
+        version = str(hash(version))
         return version
 
-    @property
-    @lru_cache()
-    def _qc_version(self) -> str:
-        """Return server QC data version.
-
-        The versioning of QC Data on the server is determined by the hash of
-        the tuple of sorted QC data objects ids.
-
-        :return: data version
-        """
-        mqc_ids = [
-            item.id
-            for item in self.collection.data.filter(
-                type="data:multiqc",
-                status="OK",
-                entity__isnull=False,
-                ordering="id",
-                fields=["id", "entity__id"],
-            )
-        ]
-        if not mqc_ids:
-            raise ValueError(
-                f"Collection {self.collection.name} has no samples with MultiQC data!"
-            )
-
-        return str(hash(tuple(mqc_ids)))
-
     @property
     @lru_cache()
     def _data_version(self) -> str:
@@ -292,14 +282,8 @@ class BaseTables(abc.ABC):
         if data is None:
             if data_type == self.META:
                 data = self._download_metadata()
-            elif data_type == self.QC:
-                data = self._download_qc()
             else:
-                # XXX: When dropping support for Python 3.6 the below
-                # statement can be replaced with:
-                # data = asyncio.run(self._download_data(data_type))
-                loop = asyncio.get_event_loop()
-                data = loop.run_until_complete(self._download_data(data_type))
+                data = asyncio.run(self._download_data(data_type))
 
             save_pickle(data, self._cache_file(data_type))
         return data
@@ -308,47 +292,47 @@ class BaseTables(abc.ABC):
         """Return full cache file path."""
         if data_type == self.META:
             version = self._metadata_version
-        elif data_type == self.QC:
-            version = self._qc_version
         else:
             version = self._data_version
 
         cache_file = f"{self.collection.slug}_{data_type}_{version}.pickle"
         return os.path.join(self.cache_dir, cache_file)
 
-    def _get_descriptors(self) -> pd.DataFrame:
-        descriptors = []
-        for sample in self._samples:
-            sample.descriptor["sample_id"] = sample.id
-            descriptors.append(sample.descriptor)
+    def _get_annotations(self) -> pd.DataFrame:
+        TYPE_TO_DTYPE = {
+            "STRING": str,
+            # Pandas cannot cast NaN's to int, but it can cast them
+            # to pd.Int64Dtype
+            "INTEGER": pd.Int64Dtype(),
+            "DECIMAL": float,
+            "DATE": "datetime64[ns]",
+        }
 
-        df = pd.json_normalize(descriptors).set_index("sample_id")
-
-        # Keep only numeric / string types:
-        column_types = {}
-        prefix = "XXX"
-        for (schema, _, path) in iterate_schema(
-            sample.descriptor, sample.descriptor_schema.schema, path=prefix
+        sample_data = defaultdict(dict)
+        sample_dtypes = defaultdict(dict)
+        # Getting annotation values for large collection breaks the server.
+        # Instead make smaller queries for a single batch of samples
+        batch_size = 100
+        for i in self.tqdm(
+            range(0, len(self._samples), batch_size), desc="Downloading annotations"
         ):
-            field_type = schema["type"]
-            field_name = path[len(prefix) + 1 :]
+            batch_sample_ids = [s.id for s in self._samples[i : i + batch_size]]
+            avs = self.resolwe.annotation_value.filter(
+                entity__in=batch_sample_ids,
+            )
+            for ann_value in avs:
+                sample_data[ann_value.sample.id][str(ann_value.field)] = ann_value.value
+                sample_dtypes[ann_value.sample.id][str(ann_value.field)] = (
+                    TYPE_TO_DTYPE[ann_value.field.type.upper()]
+                )
 
-            # This can happen if this filed has None value in all descriptors
-            if field_name not in df:
-                continue
+        annotations = []
+        for sample in self._samples:
+            data = sample_data.get(sample.id, {})
+            dtypes = sample_dtypes.get(sample.id, {})
+            annotations.append(pd.DataFrame(data, index=[sample.id]).astype(dtypes))
 
-            if field_type == "basic:string:":
-                column_types[field_name] = str
-            elif field_type == "basic:integer:":
-                # Pandas cannot cast NaN's to int, but it can cast them
-                # to pd.Int64Dtype
-                column_types[field_name] = pd.Int64Dtype()
-            elif field_type == "basic:decimal:":
-                column_types[field_name] = float
-
-        df = df[column_types.keys()].astype(column_types)
-
-        return df
+        return pd.concat(annotations, axis=0)
 
     def _get_relations(self) -> pd.DataFrame:
         relations = pd.DataFrame(index=[s.id for s in self._samples])
@@ -447,9 +431,9 @@ class BaseTables(abc.ABC):
         """Download samples metadata and transform into table."""
         meta = pd.DataFrame(None, index=[s.id for s in self._samples])
 
-        # Add descriptors metadata
-        descriptors = self._get_descriptors()
-        meta = meta.merge(descriptors, how="left", left_index=True, right_index=True)
+        # Add annotations metadata
+        annotations = self._get_annotations()
+        meta = meta.merge(annotations, how="left", left_index=True, right_index=True)
 
         # Add relations metadata
         relations = self._get_relations()
@@ -464,10 +448,6 @@ class BaseTables(abc.ABC):
 
         return meta
 
-    @abc.abstractmethod
-    def _download_qc(self) -> pd.DataFrame:
-        pass
-
     def _get_data_uri(self, data: Data, data_type: str) -> str:
         field_name = self.data_type_to_field_name[data_type]
         files = data.files(field_name=field_name)
@@ -488,7 +468,22 @@ class BaseTables(abc.ABC):
             auth=self.resolwe.auth,
         )
         response.raise_for_status()
-        return json.loads(response.content.decode("utf-8"))
+        uri_to_url = json.loads(response.content.decode("utf-8"))
+
+        def resolve_url(url):
+            """
+            Resolve url.
+
+            In case files are stored locally on a server, a local path
+            is provided. Url has to be prepended with self.resolwe.url.
+            """
+            if not urlparse(url).scheme:
+                return urljoin(self.resolwe.url, url)
+            return url
+
+        uri_to_url = {uri: resolve_url(url) for uri, url in uri_to_url.items()}
+
+        return uri_to_url
 
     @abc.abstractmethod
     def _parse_file(self, file_obj, sample_id, data_type):
@@ -505,7 +500,7 @@ class BaseTables(abc.ABC):
         return sample_data
 
     async def _download_data(self, data_type: str) -> pd.DataFrame:
-        """Download data files and marge them into a pandas DataFrame.
+        """Download data files and merge them into a pandas DataFrame.
 
         During normal download of a single file a signed url is created on AWS
         and user is than redirected from Genialis server to the signed url.
@@ -523,7 +518,7 @@ class BaseTables(abc.ABC):
         :param data_type: data type
         :return: table with data, features in columns, samples in rows
         """
-        samples_data = []
+        df = None
         for i in self.tqdm(
             range(0, len(self._data), EXP_ASYNC_CHUNK_SIZE),
             desc="Downloading data",
@@ -541,14 +536,17 @@ class BaseTables(abc.ABC):
             source_urls = self._get_data_urls(uri_to_id.keys())
             urls_ids = [(url, uri_to_id[uri]) for uri, url in source_urls.items()]
 
-            async with aiohttp.ClientSession() as session:
+            async with aiohttp.ClientSession(
+                cookies=self.resolwe.auth.cookies
+            ) as session:
                 futures = [
                     self._download_file(url, session, id_, data_type)
                     for url, id_ in urls_ids
                 ]
                 data = await asyncio.gather(*futures)
-                samples_data.extend(data)
+                data = pd.concat(data, axis=1)
+            df = pd.concat([df, data], axis=1)
 
-        df = pd.concat(samples_data, axis=1).T.sort_index().sort_index(axis=1)
+        df = df.T.sort_index().sort_index(axis=1)
         df.index.name = "sample_id"
         return df
