@@ -6,12 +6,14 @@ MATables
 
 .. autoclass:: MATables
     :members:
+    :inherited-members:
 
     .. automethod:: __init__
 
 """
+
 from functools import lru_cache
-from typing import Callable, List, Optional
+from typing import Callable, Optional
 
 import pandas as pd
 
