@@ -6,211 +6,30 @@ RNATables
 
 .. autoclass:: RNATables
     :members:
+    :inherited-members:
 
     .. automethod:: __init__
 
 """
+
 import os
 import warnings
+from collections import Counter
 from functools import lru_cache
 from typing import Callable, Dict, List, Optional
 
+import numpy as np
 import pandas as pd
 
 from resdk.resources import Collection, Data
+from resdk.tables.qc import QCTables
 from resdk.utils.table_cache import load_pickle, save_pickle
 
 from .base import BaseTables
-from .utils import batch_download
 
 CHUNK_SIZE = 1000
 
 
-MQC_GENERAL_COLUMNS = [
-    {
-        "name": "FastQC (raw)_mqc-generalstats-fastqc_raw-total_sequences",
-        "slug": "total_read_count_raw",
-        "type": "Int64",
-        "agg_func": "sum",
-    },
-    {
-        "name": "FastQC (trimmed)_mqc-generalstats-fastqc_trimmed-total_sequences",
-        "slug": "total_read_count_trimmed",
-        "type": "Int64",
-        "agg_func": "sum",
-    },
-    {
-        "name": "FastQC (raw)_mqc-generalstats-fastqc_raw-percent_gc",
-        "slug": "gc_content_raw",
-        "type": "float64",
-        "agg_func": "mean",
-    },
-    {
-        "name": "FastQC (trimmed)_mqc-generalstats-fastqc_trimmed-percent_gc",
-        "slug": "gc_content_trimmed",
-        "type": "float64",
-        "agg_func": "mean",
-    },
-    {
-        "name": "FastQC (raw)_mqc-generalstats-fastqc_raw-percent_duplicates",
-        "slug": "seq_duplication_raw",
-        "type": "float64",
-        "agg_func": "mean",
-    },
-    {
-        "name": "FastQC (trimmed)_mqc-generalstats-fastqc_trimmed-percent_duplicates",
-        "slug": "seq_duplication_trimmed",
-        "type": "float64",
-        "agg_func": "mean",
-    },
-    {
-        "name": "FastQC (raw)_mqc-generalstats-fastqc_raw-avg_sequence_length",
-        "slug": "avg_seq_length_raw",
-        "type": "float64",
-        "agg_func": "mean",
-    },
-    {
-        "name": "FastQC (trimmed)_mqc-generalstats-fastqc_trimmed-avg_sequence_length",
-        "slug": "avg_seq_length_trimmed",
-        "type": "float64",
-        "agg_func": "mean",
-    },
-    {
-        "name": "STAR_mqc-generalstats-star-uniquely_mapped_percent",
-        "slug": "mapped_reads_percent",
-        "type": "float64",
-        "agg_func": "mean",
-    },
-    {
-        "name": "STAR_mqc-generalstats-star-uniquely_mapped",
-        "slug": "mapped_reads",
-        "type": "Int64",
-        "agg_func": "sum",
-    },
-    {
-        "name": "STAR (Globin)_mqc-generalstats-star_globin-uniquely_mapped_percent",
-        "slug": "mapped_reads_percent_globin",
-        "type": "float64",
-        "agg_func": "mean",
-    },
-    {
-        "name": "STAR (Globin)_mqc-generalstats-star_globin-uniquely_mapped",
-        "slug": "mapped_reads_globin",
-        "type": "Int64",
-        "agg_func": "sum",
-    },
-    {
-        "name": "STAR (rRNA)_mqc-generalstats-star_rrna-uniquely_mapped_percent",
-        "slug": "mapped_reads_percent_rRNA",
-        "type": "float64",
-        "agg_func": "mean",
-    },
-    {
-        "name": "STAR (rRNA)_mqc-generalstats-star_rrna-uniquely_mapped",
-        "slug": "mapped_reads_rRNA",
-        "type": "Int64",
-        "agg_func": "sum",
-    },
-    {
-        "name": "featureCounts_mqc-generalstats-featurecounts-percent_assigned",
-        "slug": "fc_assigned_reads_percent",
-        "type": "float64",
-        "agg_func": "mean",
-    },
-    {
-        "name": "featureCounts_mqc-generalstats-featurecounts-Assigned",
-        "slug": "fc_assigned_reads",
-        "type": "Int64",
-        "agg_func": "sum",
-    },
-    {
-        "name": "Salmon_mqc-generalstats-salmon-percent_mapped",
-        "slug": "salmon_assigned_reads_percent",
-        "type": "float64",
-        "agg_func": "mean",
-    },
-    {
-        "name": "Salmon_mqc-generalstats-salmon-num_mapped",
-        "slug": "salmon_assigned_reads",
-        "type": "Int64",
-        "agg_func": "sum",
-    },
-    {
-        "name": "QoRTs_mqc-generalstats-qorts-Genes_PercentWithNonzeroCounts",
-        "slug": "nonzero_count_features_percent",
-        "type": "float64",
-        "agg_func": "mean",
-    },
-    {
-        "name": "QoRTs_mqc-generalstats-qorts-NumberOfChromosomesCovered",
-        "slug": "contigs_covered",
-        "type": "Int64",
-        "agg_func": "mean",
-    },
-    {
-        "slug": "strandedness_code",
-        "type": "string",
-    },
-    {
-        "slug": "genome_build",
-        "type": "string",
-    },
-]
-
-
-def multiqc_general_stats_parser(file_object, name):
-    """Parse "multiqc_general_stats.txt" file."""
-    df = pd.read_csv(file_object, sep="\t", index_col=0)
-
-    # Keep only specified columns:
-    df = df[
-        [
-            column.get("name", "")
-            for column in MQC_GENERAL_COLUMNS
-            if column.get("name", "") in df.columns
-        ]
-    ]
-    # Rename
-    df = df.rename(
-        columns={
-            column.get("name", ""): column["slug"]
-            for column in MQC_GENERAL_COLUMNS
-            if column.get("name", "") in df.columns
-        }
-    )
-
-    # Perform aggregation
-    series = df.agg(
-        {
-            column["slug"]: column["agg_func"]
-            for column in MQC_GENERAL_COLUMNS
-            if column["slug"] in df.columns
-        }
-    )
-    series.name = name
-    return series
-
-
-def multiqc_strand_parser(file_object, name):
-    """Parse "multiqc_library_strandedness.txt" file."""
-    df = pd.read_csv(file_object, sep="\t", index_col=0)
-    return pd.Series(
-        df["Strandedness code"].iloc[0],
-        index=["strandedness_code"],
-        name=name,
-    )
-
-
-def multiqc_build_parser(file_object, name):
-    """Parse "multiqc_sample_info.txt" file."""
-    df = pd.read_csv(file_object, sep="\t", index_col=0)
-    return pd.Series(
-        df["Genome Build"].iloc[0],
-        index=["genome_build"],
-        name=name,
-    )
-
-
 class RNATables(BaseTables):
     """A helper class to fetch collection's expression and meta data.
 
@@ -354,7 +173,7 @@ class RNATables(BaseTables):
 
         .. code-block:: python
 
-            exp = exp.rename(columns=tables.id_to_symbol)
+            exp = exp.rename(columns=tables.readable_columns)
 
         :return: dict with gene ids as keys and gene symbols as values
         """
@@ -364,18 +183,36 @@ class RNATables(BaseTables):
         if not self.gene_ids:
             raise ValueError("Expression data must be used before!")
 
-        return self._mapping(self.gene_ids, source, species)
+        mapping = self._mapping(self.gene_ids, source, species)
+        if len(mapping) < len(self.gene_ids):
+            missing = list(set(self.gene_ids) - set(mapping))
+            missing_str = (
+                "(" + ", ".join(missing[:5]) + (", ...)" if len(missing) > 5 else ")")
+            )
+            warnings.warn(
+                f"Symbols for {len(missing)} gene IDs were not found. ({missing_str})"
+                "Missing symbols will be set to empty string.",
+                UserWarning,
+            )
+            mapping = {id_: mapping.get(id_, np.nan) for id_ in self.gene_ids}
+        return mapping
 
     @property
     @lru_cache()
-    def id_to_symbol(self) -> Dict[str, str]:
-        """Map of source gene ids to symbols."""
-        warnings.warn(
-            "Attribute `id_to_symbol` will be removed in Q1 of 2022. "
-            "Use `readable_columns` instead.",
-            DeprecationWarning,
-        )
-        return self.readable_columns
+    def build(self) -> str:
+        """Get build."""
+        builds = Counter([d.output.get("build") for d in self._data])
+
+        if len(builds) == 0:
+            raise ValueError("Cannot determine build, no data found.")
+        elif len(builds) > 1:
+            builds_str = ", ".join(k for k in builds.keys())
+            warnings.warn(
+                f"Cannot determine build, multiple builds found: {builds_str}."
+            )
+
+        # Return the only / most common build
+        return builds.most_common(1)[0][0]
 
     @property
     @lru_cache()
@@ -402,67 +239,19 @@ class RNATables(BaseTables):
         """Return full cache file path."""
         if data_type == self.META:
             version = self._metadata_version
-        elif data_type == self.QC:
-            version = self._qc_version
         else:
             version = self._data_version
 
         cache_file = f"{self.collection.slug}_{data_type}_{self.expression_source}_{self.expression_process_slug}_{version}.pickle"
         return os.path.join(self.cache_dir, cache_file)
 
-    def _download_qc(self) -> pd.DataFrame:
-        """Download sample QC data and transform into table.
-
-        QC info can be sourced from multiple files and multiple columns
-        in those files.
-        """
-        mqc_db = {}
-        for mqc in self.collection.data.filter(
-            type="data:multiqc",
-            status="OK",
-            ordering="created",
-            entity__isnull=False,
-            fields=["id", "entity__id"],
-        ):
-            if mqc.sample.id in mqc_db:
-                warnings.warn(
-                    f"Sample with ID={mqc.sample.id} has multiple MultiQC Data. "
-                    "Using the latest one.",
-                    UserWarning,
-                )
-            mqc_db[mqc.sample.id] = {
-                "mqc_id": mqc.id,
-                "uri_general": f"{mqc.id}/multiqc_data/multiqc_general_stats.txt",
-                "uri_strand": f"{mqc.id}/multiqc_data/multiqc_library_strandedness.txt",
-                "uri_build": f"{mqc.id}/multiqc_data/multiqc_sample_info.txt",
-            }
-
-        df = pd.DataFrame(index=[sample.id for sample in self._samples])
-
-        parsers = {
-            "uri_general": multiqc_general_stats_parser,
-            "uri_strand": multiqc_strand_parser,
-            "uri_build": multiqc_build_parser,
-        }
-        for type_, parser in parsers.items():
-            uris = [item[type_] for item in mqc_db.values()]
-            qc = batch_download(self.resolwe, uris, parser)
-            qc = qc.rename(index={value[type_]: key for key, value in mqc_db.items()})
-            df = df.merge(qc, how="left", left_index=True, right_index=True)
-
-        # Cast to correct dtype
-        column_types = {
-            c["slug"]: c["type"] for c in MQC_GENERAL_COLUMNS if c["slug"] in df.columns
-        }
-        df = df[column_types.keys()].astype(column_types)
-
-        return df
-
     def _parse_file(self, file_obj, sample_id, data_type):
         """Parse file object and return a one DataFrame line."""
         sample_data = pd.read_csv(file_obj, sep="\t", compression="gzip")
         sample_data = sample_data.set_index("Gene")["Expression"]
         sample_data.name = sample_id
+        # Optimize memory usage, float32 and int32 will suffice.
+        sample_data = sample_data.astype("int32" if data_type == self.RC else "float32")
         return sample_data
 
     async def _download_data(self, data_type: str) -> pd.DataFrame:
@@ -472,6 +261,13 @@ class RNATables(BaseTables):
         df.attrs["exp_type"] = (
             "rc" if data_type == self.RC else self._data[0].output["exp_type"]
         )
+        try:
+            df.attrs["build"] = self.build
+        except ValueError:
+            # In rare cases, it can happen that Collection has Data with
+            # different builds but same source. In such case, just skip
+            # assigning the build.
+            pass
         return df
 
     def _mapping(self, ids: List[str], source: str, species: str) -> Dict[str, str]:
@@ -507,3 +303,17 @@ class RNATables(BaseTables):
             )
             mapping.update({f.feature_id: f.name for f in features})
         return mapping
+
+    @property
+    @lru_cache()
+    def qc(self) -> pd.DataFrame:
+        """Return QC table as a pandas DataFrame object.
+
+        :return: table of QC values
+        """
+        warnings.warn(
+            "Method `RNATables(<collection>).qc` will be deprecated in next major "
+            "release. Use QCTables(<collection>).rnaseq to get the same results.",
+            DeprecationWarning,
+        )
+        return QCTables(self.collection).rnaseq
