@@ -2,23 +2,23 @@
 Unit tests for resdk/resources/collection.py file.
 """
 
-
 import unittest
 
 from mock import MagicMock
 
 from resdk.resources.collection import Collection
 
+from .utils import server_resource
+
 
 class TestCollection(unittest.TestCase):
     def test_create_group(self):
-        collection = Collection(id=1, resolwe=MagicMock())
-        collection.id = 1  # this is overriden when initialized
+        collection = server_resource(Collection, id=1, resolwe=MagicMock())
 
         # only samples
         collection.create_group_relation(category="replicates", samples=[1, 2, 3])
         collection.resolwe.relation.create.assert_called_with(
-            collection=1,
+            collection=collection,
             type="group",
             category="replicates",
             partitions=[
@@ -37,7 +37,7 @@ class TestCollection(unittest.TestCase):
             labels=["first", "second", "third"],
         )
         collection.resolwe.relation.create.assert_called_with(
-            collection=1,
+            collection=collection,
             type="group",
             category="replicates",
             partitions=[
@@ -59,13 +59,12 @@ class TestCollection(unittest.TestCase):
         collection.resolwe.relation.create.reset_mock()
 
     def test_create_compare(self):
-        collection = Collection(id=1, resolwe=MagicMock())
-        collection.id = 1  # this is overriden when initialized
+        collection = server_resource(Collection, id=1, resolwe=MagicMock())
 
         # only samples
         collection.create_compare_relation(category="case-control", samples=[1, 2])
         collection.resolwe.relation.create.assert_called_with(
-            collection=1,
+            collection=collection,
             type="compare",
             category="case-control",
             partitions=[
@@ -81,7 +80,7 @@ class TestCollection(unittest.TestCase):
             category="case-control", samples=[1, 2], labels=["case", "control"]
         )
         collection.resolwe.relation.create.assert_called_with(
-            collection=1,
+            collection=collection,
             type="compare",
             category="case-control",
             partitions=[
@@ -102,13 +101,12 @@ class TestCollection(unittest.TestCase):
         collection.resolwe.relation.create.reset_mock()
 
     def test_create_series(self):
-        collection = Collection(id=1, resolwe=MagicMock())
-        collection.id = 1  # this is overriden when initialized
+        collection = server_resource(Collection, id=1, resolwe=MagicMock())
 
         # only samples
         collection.create_series_relation(category="time-series", samples=[1, 2, 3])
         collection.resolwe.relation.create.assert_called_with(
-            collection=1,
+            collection=collection,
             type="series",
             category="time-series",
             partitions=[
@@ -125,7 +123,7 @@ class TestCollection(unittest.TestCase):
             category="time-series", samples=[1, 2, 3], labels=["0Hr", "2Hr", "4Hr"]
         )
         collection.resolwe.relation.create.assert_called_with(
-            collection=1,
+            collection=collection,
             type="series",
             category="time-series",
             partitions=[
@@ -140,7 +138,7 @@ class TestCollection(unittest.TestCase):
             category="time-series", samples=[1, 2, 3], positions=[10, 20, 30]
         )
         collection.resolwe.relation.create.assert_called_with(
-            collection=1,
+            collection=collection,
             type="series",
             category="time-series",
             partitions=[
@@ -162,13 +160,12 @@ class TestCollection(unittest.TestCase):
         collection.resolwe.relation.create.reset_mock()
 
     def test_create_background(self):
-        collection = Collection(id=1, resolwe=MagicMock())
-        collection.id = 1  # this is overriden when initialized
+        collection = server_resource(Collection, id=1, resolwe=MagicMock())
 
         # only samples
         collection.create_background_relation("background 1", "sample_1", ["sample_2"])
         collection.resolwe.relation.create.assert_called_with(
-            collection=1,
+            collection=collection,
             type="background",
             category="background 1",
             partitions=[
