@@ -2,7 +2,6 @@
 Unit tests for resdk/query.py file.
 """
 
-
 import unittest
 from collections import defaultdict
 
@@ -131,21 +130,31 @@ class TestResolweQuery(unittest.TestCase):
 
     def test_add_filter(self):
         query = MagicMock(
-            spec=ResolweQuery, _filters=defaultdict(list, {"slug": ["test"]})
+            spec=ResolweQuery,
+            _filters=defaultdict(list, {"slug": ["test"]}),
+            _dehydrate_resources=MagicMock(return_value=1),
+            _non_string_iterable=MagicMock(return_value=False),
         )
         query.resource.query_method = "GET"
         ResolweQuery._add_filter(query, {"id": 1})
         self.assertEqual(dict(query._filters), {"slug": ["test"], "id": [1]})
 
-        query = MagicMock(spec=ResolweQuery, _filters={"slug": "test"})
+        query = MagicMock(
+            spec=ResolweQuery,
+            _filters={"slug": "test"},
+            _dehydrate_resources=MagicMock(return_value=1),
+            _non_string_iterable=MagicMock(return_value=False),
+        )
         query.resource.query_method = "POST"
         ResolweQuery._add_filter(query, {"id": 1})
         self.assertEqual(query._filters, {"slug": "test", "id": 1})
 
-        query = MagicMock(spec=ResolweQuery, _filters={"slug": "test"})
-        query.resource.query_method = "POST"
-        ResolweQuery._add_filter(query, {"sample": "my_sample"})
-        self.assertEqual(query._filters, {"slug": "test", "entity": "my_sample"})
+        query = MagicMock(
+            spec=ResolweQuery,
+            _filters={"slug": "test"},
+            _dehydrate_resources=MagicMock(return_value=2),
+            _non_string_iterable=MagicMock(return_value=False),
+        )
 
     def test_compose_filters(self):
         query = MagicMock(spec=ResolweQuery)
