@@ -2,8 +2,10 @@
 Unit tests for resdk/resolwe.py file.
 """
 
-
+import json
 import os
+import shutil
+import tempfile
 import unittest
 
 import requests
@@ -13,8 +15,11 @@ from slumber.exceptions import SlumberHttpBaseException
 from resdk.exceptions import ResolweServerError, ValidationError
 from resdk.resolwe import ResAuth, Resolwe, ResolweResource
 from resdk.resources import Collection, Data, Process
+from resdk.resources.fields import DataSource
 from resdk.uploader import Uploader
 
+from .utils import server_resource
+
 BASE_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
 
 
@@ -292,7 +297,6 @@ class TestRun(unittest.TestCase):
     @patch("resdk.resolwe.Data")
     @patch("resdk.resolwe.Resolwe", spec=True)
     def test_file_processing(self, resolwe_mock, data_mock):
-
         resolwe_mock.api = MagicMock(
             **{
                 "process.get.return_value": self.process_mock,
@@ -318,8 +322,7 @@ class TestRun(unittest.TestCase):
 
     @patch("resdk.resolwe.Resolwe", spec=True)
     def test_dehydrate_data(self, resolwe_mock):
-        data_obj = Data(id=1, resolwe=MagicMock())
-        data_obj.id = 1  # this is overriden when initialized
+        data_obj = server_resource(Data, id=1, resolwe=MagicMock())
         process = self.process_mock
 
         result = Resolwe._process_inputs(resolwe_mock, {"genome": data_obj}, process)
@@ -339,7 +342,7 @@ class TestRun(unittest.TestCase):
         Resolwe.run(
             self=resolwe_mock,
             slug="process-slug",
-            collection=Collection(id=1, resolwe=MagicMock()),
+            collection=server_resource(Collection, id=1, resolwe=MagicMock()),
         )
         resolwe_mock.api.data.post.assert_called_once_with(
             {
@@ -371,7 +374,11 @@ class TestRun(unittest.TestCase):
         )
         # Confirm that no files to upload in input:
         self.assertEqual(resolwe_mock.uploader.upload.call_count, 0)
-        data_mock.assert_called_with(data="some_data", resolwe=resolwe_mock)
+        data_mock.assert_called_with(
+            data="some_data",
+            resolwe=resolwe_mock,
+            initial_data_source=DataSource.SERVER,
+        )
         self.assertEqual(data, "Data object")
 
 
@@ -438,119 +445,162 @@ class TestDownload(unittest.TestCase):
             "auth": MagicMock(),
             "logger": MagicMock(),
         }
+        self.tmp_dir = tempfile.mkdtemp()
+
+    def tearDown(self):
+        if os.path.isdir(self.tmp_dir):
+            shutil.rmtree(self.tmp_dir)
 
-    @patch("resdk.resolwe.os")
     @patch("resdk.resolwe.Resolwe", spec=True)
-    def test_fail_if_bad_dir(self, resolwe_mock, os_mock):
+    def test_fail_if_bad_dir(self, resolwe_mock):
         resolwe_mock.configure_mock(**self.config)
-        os_mock.path.isdir.return_value = False
 
         message = "Download directory does not exist: .*"
         with self.assertRaisesRegex(ValueError, message):
-            Resolwe._download_files(resolwe_mock, self.file_list)
+            Resolwe._download_files(
+                resolwe_mock, files=self.file_list, download_dir="/does/not/exist/"
+            )
 
-    @patch("resdk.resolwe.os")
     @patch("resdk.resolwe.Resolwe", spec=True)
-    def test_empty_file_list(self, resolwe_mock, os_mock):
+    def test_empty_file_list(self, resolwe_mock):
         resolwe_mock.configure_mock(**self.config)
 
-        Resolwe._download_files(resolwe_mock, [])
+        Resolwe._download_files(resolwe_mock, files=[], download_dir=self.tmp_dir)
 
         resolwe_mock.logger.info.assert_called_once_with("No files to download.")
 
-    @patch("resdk.resolwe.os")
     @patch("resdk.resolwe.Resolwe", spec=True)
-    def test_bad_response(self, resolwe_mock, os_mock):
+    def test_bad_response(self, resolwe_mock):
         resolwe_mock.configure_mock(**self.config)
-        os_mock.path.isfile.return_value = True
         response = {"raise_for_status.side_effect": Exception("abc")}
-        resolwe_mock.session.get.return_value = MagicMock(ok=False, **response)
+        sizes = [{"name": "file.txt", "size": 1, "type": "file", "md5": 123}]
+        resolwe_mock.session.get.side_effect = [
+            MagicMock(content=json.dumps(sizes)),
+            MagicMock(ok=False, **response),
+        ]
 
         with self.assertRaisesRegex(Exception, "abc"):
-            Resolwe._download_files(resolwe_mock, self.file_list[:1])
+            Resolwe._download_files(
+                resolwe_mock,
+                files=self.file_list[:1],
+                download_dir=self.tmp_dir,
+            )
         self.assertEqual(resolwe_mock.logger.info.call_count, 2)
 
-    @patch("resdk.resolwe.open")
-    @patch("resdk.resolwe.os")
     @patch("resdk.resolwe.Resolwe", spec=True)
-    def test_good_response(self, resolwe_mock, os_mock, open_mock):
+    def test_good_response(self, resolwe_mock):
         resolwe_mock.configure_mock(**self.config)
-        os_mock.path.isfile.return_value = True
 
-        resolwe_mock.session.get.return_value = MagicMock(
-            ok=True, **{"iter_content.return_value": range(3)}
+        size_file1 = [
+            {
+                "name": "file.txt",
+                "size": 1,
+                "type": "file",
+                "md5": "e3cdf70a99c1d6890c54ad56bd4a5de1",
+            }
+        ]
+        size_file2 = [
+            {
+                "name": "file.py",
+                "size": 2,
+                "type": "file",
+                "md5": "f1a8bf29b1df09dd9082f8f8fece0839",
+            }
+        ]
+        resolwe_mock.session.get.side_effect = [
+            MagicMock(content=json.dumps(size_file1)),
+            MagicMock(ok=True, **{"iter_content.return_value": [b"11", b"12", b"13"]}),
+            MagicMock(content=json.dumps(size_file2)),
+            MagicMock(ok=True, **{"iter_content.return_value": [b"21", b"22", b"23"]}),
+        ]
+
+        Resolwe._download_files(
+            resolwe_mock,
+            files=self.file_list,
+            download_dir=self.tmp_dir,
+            show_progress=False,
         )
-
-        Resolwe._download_files(resolwe_mock, self.file_list)
         self.assertEqual(resolwe_mock.logger.info.call_count, 3)
 
-        # This asserts may seem wierd. To check what is happening behind the scenes:
-        # print(open_mock.mock_calls)
-        self.assertEqual(
-            open_mock.return_value.__enter__.return_value.write.call_count, 6
-        )
-        # Why 6? 2 files in self.file_list, each downloads 3 chunks (defined in response mock)
-
 
 class TestResAuth(unittest.TestCase):
     @patch("resdk.resolwe.ResAuth", spec=True)
     def setUp(self, auth_mock):
+        auth_mock.interactive_login_url = "https://url.com/remote-auth/"
         auth_mock.configure_mock(sessionid=None, csrftoken=None)
         self.auth_mock = auth_mock
 
-    @patch("resdk.resolwe.requests.post")
-    def test_bad_url(self, requests_post_mock):
-        requests_post_mock.side_effect = requests.exceptions.ConnectionError()
-
-        with self.assertRaisesRegex(
-            ValueError, "Server not accessible on www.abc.com. Wrong url?"
-        ):
-            ResAuth.__init__(
-                self.auth_mock, username="usr", password="pwd", url="www.abc.com"
-            )
-
-    @patch("resdk.resolwe.requests")
-    def test_bad_credentials(self, requests_mock):
-        requests_mock.post = MagicMock(return_value=MagicMock(status_code=400))
-
-        message = r"Response HTTP status code .* Invalid credentials?"
-        with self.assertRaisesRegex(ValueError, message):
-            ResAuth.__init__(
-                self.auth_mock, username="usr", password="pwd", url="www.abc.com"
-            )
-
-    @patch("resdk.resolwe.requests")
-    def test_no_csrf_token(self, requests_mock):
-        post_mock = MagicMock(status_code=200, cookies={"sessionid": 42})
-        requests_mock.post = MagicMock(return_value=post_mock)
-
-        message = "Missing sessionid or csrftoken. Invalid credentials?"
-        with self.assertRaisesRegex(Exception, message):
-            ResAuth.__init__(
-                self.auth_mock, username="usr", password="pwd", url="www.abc.com"
-            )
-
-    @patch("resdk.resolwe.requests")
-    def test_all_ok(self, requests_mock):
-        post_mock = MagicMock(
-            status_code=200, cookies={"sessionid": 42, "csrftoken": 43}
-        )
-        requests_mock.post = MagicMock(return_value=post_mock)
+    def test_init(self):
+        ResAuth.__init__(self.auth_mock, "uname", "pw", "https://url.com")
+        self.auth_mock.automatic_login.assert_called_once_with("uname", "pw")
+        self.auth_mock.interactive_login.assert_not_called()
 
+    def test_interactive_fallback_no_username(self):
+        """Only fall back to automatic login if username is not provided."""
         ResAuth.__init__(
-            self.auth_mock, username="usr", password="pwd", url="www.abc.com"
+            self.auth_mock,
+            username="test",
+            password="pass",
+            url="https://url.com",
+            interactive=True,
         )
-        self.assertEqual(self.auth_mock.sessionid, 42)
-        self.assertEqual(self.auth_mock.csrftoken, 43)
+        self.auth_mock.interactive_login.assert_not_called()
+        self.auth_mock.reset_mock()
+
+        # Attempt interactive login only when no username is given.
+        ResAuth.__init__(
+            self.auth_mock, password="pw", url="https://url.com", interactive=True
+        )
+        self.auth_mock.interactive_login.assert_called_once_with()
 
     @patch("resdk.resolwe.requests")
-    def test_public_user(self, requests_mock):
-        post_mock = MagicMock(status_code=200)
-        requests_mock.post = MagicMock(return_value=post_mock)
+    @patch("resdk.resolwe.webbrowser")
+    @patch("resdk.resolwe.print")
+    @patch("resdk.resolwe.time")
+    def test_interactive_login(
+        self, time_mock, print_mock, webbrowser_mock, requests_mock
+    ):
+        requests_mock.get.return_value = MagicMock(
+            status_code=200, **{"json.return_value": {"auth_id": "123"}}
+        )
+        requests_mock.Session.return_value.get.side_effect = [
+            MagicMock(status_code=204),
+            MagicMock(
+                status_code=200,
+                json=MagicMock(
+                    return_value={"csrftoken": "my-token", "sessionid": "my-id"}
+                ),
+            ),
+        ]
+        self.assertDictEqual(
+            ResAuth.interactive_login(self.auth_mock),
+            {"csrftoken": "my-token", "sessionid": "my-id"},
+        )
+        webbrowser_mock.open.assert_called_once_with("https://url.com/remote-auth/")
+        requests_mock.get.assert_called_once_with(
+            "https://url.com/remote-auth/auth-id/"
+        )
+        print_mock.assert_called_once()
+        time_mock.sleep.assert_called_once_with(1)
+        self.assertEqual(requests_mock.Session.return_value.get.call_count, 2)
+        requests_mock.Session.return_value.get.assert_called_with(
+            "https://url.com/remote-auth/poll/"
+        )
 
-        ResAuth.__init__(self.auth_mock, url="www.abc.com")
-        self.assertEqual(self.auth_mock.sessionid, None)
-        self.assertEqual(self.auth_mock.csrftoken, None)
+    @patch("resdk.resolwe.requests")
+    def test_automatic_login(self, requests_mock):
+        requests_mock.post.return_value = MagicMock(status_code=204)
+        requests_mock.post.return_value.cookies.get_dict.return_value = {
+            "csrftoken": "my-token",
+            "sessionid": "my-id",
+        }
+        self.auth_mock.logger = MagicMock()
+        self.auth_mock.automatic_login_url = "http://www.abc.com/saml-auth/api-login/"
+        cookies = ResAuth.automatic_login(
+            self.auth_mock, username="me", password="pass"
+        )
+        self.auth_mock.logger.info.assert_called_once()
+        self.assertDictEqual(cookies, {"csrftoken": "my-token", "sessionid": "my-id"})
 
     def test_call(self):
         res_auth = MagicMock(
@@ -560,7 +610,9 @@ class TestResAuth(unittest.TestCase):
         self.assertDictEqual(resp.headers, {"referer": "www.abc.com"})
 
         res_auth = MagicMock(
-            spec=ResAuth, sessionid="my-id", csrftoken="my-token", url="abc.com"
+            spec=ResAuth,
+            cookies={"csrftoken": "my-token", "sessionid": "my-id"},
+            url="abc.com",
         )
         resp = ResAuth.__call__(res_auth, MagicMock(headers={}))
         self.assertDictEqual(
