@@ -4,27 +4,44 @@ from time import sleep, time
 
 import pandas as pd
 import pytz
-from mock import MagicMock, patch
+from mock import MagicMock, NonCallableMagicMock, patch
 from pandas.testing import assert_frame_equal
 
+from resdk.resources import AnnotationField, AnnotationValue, Sample
 from resdk.tables import RNATables
 
+from .utils import server_resource
+
 
 class TestTables(unittest.TestCase):
     def setUp(self):
         self.resolwe = MagicMock()
         self.resolwe.url = "https://server.com"
 
-        self.sample = MagicMock()
+        self.sample = NonCallableMagicMock(spec=Sample)
         self.sample.id = 123
         self.sample.name = "Sample123"
         self.sample.modified = datetime(
             2020, 11, 1, 12, 15, 0, 0, tzinfo=pytz.UTC
         ).astimezone(pytz.timezone("Europe/Ljubljana"))
-        self.sample.descriptor_schema.schema = [
-            {"name": "PFS", "type": "basic:decimal:", "label": "P"}
+        self.af1 = server_resource(
+            AnnotationField,
+            id=1,
+            resolwe=self.resolwe,
+            name="PFS",
+            group=dict(name="general"),
+            type="DECIMAL",
+        )
+        self.av1 = server_resource(
+            AnnotationValue,
+            resolwe=self.resolwe,
+            value=1,
+            entity=self.sample,
+            field=self.af1,
+        )
+        self.resolwe.annotation_value.filter.return_value = [
+            self.av1,
         ]
-        self.sample.descriptor = {"PFS": 1}
 
         self.data = MagicMock()
         self.data.id = 12345
@@ -43,11 +60,17 @@ class TestTables(unittest.TestCase):
             2020, 9, 1, 12, 15, 0, 0, tzinfo=pytz.UTC
         ).astimezone(pytz.timezone("Europe/Ljubljana"))
 
+        self.ann_value = MagicMock()
+        self.ann_value.id = 1024
+        self.ann_value.modified = datetime(
+            2020, 9, 2, 12, 15, 0, 0, tzinfo=pytz.UTC
+        ).astimezone(pytz.timezone("Europe/Ljubljana"))
+
         self.collection = MagicMock()
         self.collection.slug = "slug"
         self.collection.name = "Name"
-        self.collection.samples.filter = self.web_request([self.sample])
-        self.collection.data.filter = self.web_request([self.data])
+        self.collection.samples.filter().iterate = self.web_request([self.sample])
+        self.collection.data.filter().iterate = self.web_request([self.data])
         self.collection.resolwe = self.resolwe
 
         self.relation = MagicMock()
@@ -102,7 +125,7 @@ class TestTables(unittest.TestCase):
         data2.id = 12345
         data2.process.slug = "process-slug2"
         data2.output.__getitem__.side_effect = {"source": "ENSEMBL"}.__getitem__
-        self.collection.data.filter = self.web_request([self.data, data2])
+        self.collection.data.filter().iterate = self.web_request([self.data, data2])
 
         with self.assertRaisesRegex(ValueError, r"Expressions of all samples.*"):
             RNATables(self.collection)
@@ -112,7 +135,7 @@ class TestTables(unittest.TestCase):
         data2.id = 12345
         data2.process.slug = "process-slug"
         data2.output.__getitem__.side_effect = {"source": "GENCODE"}.__getitem__
-        self.collection.data.filter = self.web_request([self.data, data2])
+        self.collection.data.filter().iterate = self.web_request([self.data, data2])
 
         with self.assertRaisesRegex(ValueError, r"Alignment of all samples.*"):
             RNATables(self.collection)
@@ -193,17 +216,17 @@ class TestTables(unittest.TestCase):
         self.assertIs(rc, self.expressions_df)
 
     @patch.object(RNATables, "_mapping")
-    def test_id_to_symbol(self, mapping_mock):
+    def test_readable_columns(self, mapping_mock):
         mapping_mock.side_effect = self.web_request(self.gene_map)
 
         ct = RNATables(self.collection)
         with self.assertRaises(ValueError):
-            mapping = ct.id_to_symbol
+            mapping = ct.readable_columns
 
         ct = RNATables(self.collection)
         ct.gene_ids = ["ENSG001", "ENSG002", "ENSG003"]
         t = time()
-        mapping = ct.id_to_symbol
+        mapping = ct.readable_columns
         self.assertTrue(time() - t > 0.1)
         mapping_mock.assert_called_with(
             ["ENSG001", "ENSG002", "ENSG003"], "ENSEMBL", "Homo sapiens"
@@ -211,28 +234,24 @@ class TestTables(unittest.TestCase):
         self.assertIs(mapping, self.gene_map)
 
         # test if use case works
-        new_exp = self.expressions_df.rename(columns=ct.id_to_symbol)
+        new_exp = self.expressions_df.rename(columns=ct.readable_columns)
         self.assertListEqual(new_exp.columns.tolist(), ["GA", "GB", "GC"])
 
         # use cache
         t = time()
-        mapping = ct.id_to_symbol
+        mapping = ct.readable_columns
         self.assertTrue(time() - t < 0.1)
         self.assertIs(mapping, self.gene_map)
 
-    @patch("resdk.tables.base.clear_cache_dir_resdk")
-    def test_clear_cache(self, clear_mock):
-        RNATables.clear_cache()
-        clear_mock.assert_called()
-
     def test_metadata_version(self):
         self.collection.samples.get = self.web_request(self.sample)
         self.collection.relations.get = self.web_request(self.relation)
         self.collection.data.get = self.web_request(self.orange_data)
+        self.collection.resolwe.annotation_value.get = self.web_request(self.ann_value)
 
         ct = RNATables(self.collection)
         version = ct._metadata_version
-        self.assertEqual(version, "2020-11-01T12:15:00Z")
+        self.assertEqual(version, str(hash("2020-11-01T12:15:00Z")))
 
         # use cache
         t = time()
@@ -244,23 +263,6 @@ class TestTables(unittest.TestCase):
         with self.assertRaises(ValueError):
             version = ct1._metadata_version
 
-    def test_qc_version(self):
-        self.collection.data.filter = self.web_request([self.data])
-
-        ct = RNATables(self.collection)
-        version = ct._qc_version
-        self.assertEqual(version, str(hash(tuple([12345]))))
-
-        # use cache
-        t = time()
-        version = ct._qc_version
-        self.assertTrue(time() - t < 0.1)
-
-        self.collection.data.filter = self.web_request([])
-        ct1 = RNATables(self.collection)
-        with self.assertRaises(ValueError):
-            version = ct1._qc_version
-
     def test_data_version(self):
         ct = RNATables(self.collection)
         version = ct._data_version
@@ -271,7 +273,7 @@ class TestTables(unittest.TestCase):
         version = ct._data_version
         self.assertTrue(time() - t < 0.1)
 
-        self.collection.data.filter = MagicMock(return_value=[])
+        self.collection.data.filter().iterate = MagicMock(return_value=[])
         ct = RNATables(self.collection)
         with self.assertRaises(ValueError):
             version = ct._data_version
@@ -289,12 +291,13 @@ class TestTables(unittest.TestCase):
         self.collection.samples.get = self.web_request(self.sample)
         self.collection.relations.get = self.web_request(self.relation)
         self.collection.data.get = self.web_request(self.orange_data)
+        self.collection.resolwe.annotation_value.get = self.web_request(self.ann_value)
         ct = RNATables(self.collection)
         data = ct._load_fetch(RNATables.META)
         self.assertIs(data, self.metadata_df)
         save_mock.assert_called_with(
             self.metadata_df,
-            "/tmp/resdk/slug_meta_None_None_2020-11-01T12:15:00Z.pickle",
+            f"/tmp/resdk/slug_meta_None_None_{str(hash('2020-11-01T12:15:00Z'))}.pickle",
         )
 
         save_mock.reset_mock()
@@ -324,12 +327,12 @@ class TestTables(unittest.TestCase):
 
     def test_get_descriptors(self):
         ct = RNATables(self.collection)
-        descriptors = ct._get_descriptors()
+        anns = ct._get_annotations()
 
-        expected = pd.DataFrame([1], columns=["PFS"], index=[123], dtype=float)
+        expected = pd.DataFrame([1], columns=["general.PFS"], index=[123], dtype=float)
         expected.index.name = "sample_id"
 
-        assert_frame_equal(descriptors, expected)
+        assert_frame_equal(anns, expected, check_names=False)
 
     def test_get_relations(self):
         ct = RNATables(self.collection)
@@ -361,11 +364,11 @@ class TestTables(unittest.TestCase):
 
         assert_frame_equal(orange_data, expected)
 
-    @patch.object(RNATables, "_get_descriptors")
+    @patch.object(RNATables, "_get_annotations")
     @patch.object(RNATables, "_get_relations")
     @patch.object(RNATables, "_get_orange_data")
-    def test_download_metadata(self, orange_mock, relations_mock, descriptors_mock):
-        descriptors_mock.return_value = self.metadata_df
+    def test_download_metadata(self, orange_mock, relations_mock, annotations_mock):
+        annotations_mock.return_value = self.metadata_df
         relations_mock.return_value = pd.DataFrame(
             [["A"]], index=[123], columns=["Replicate"]
         )
